@@ -2,11 +2,20 @@ import pytest
 
 from pytest_djangoapp import configure_djangoapp_plugin
 
+
+def hook(settings):
+    apps = settings['INSTALLED_APPS']
+    apps.remove('sitetree.tests.testapp')
+    apps.append('sitetree.tests.testapp.conf.MyAppConfig')
+    return settings
+
+
 pytest_plugins = configure_djangoapp_plugin(
     settings=dict(
         SITETREE_CLS='sitetree.tests.testapp.mysitetree.MySiteTree',
     ),
     admin_contrib=True,
+    settings_hook=hook
 )
 
 
